.contextMenu {
  position: fixed;
  /* xterm accessibility tree has z-index 100 and we need to be in front of it
     * to be able to handle mouse events.
     */
  z-index: 101;
  background: white;
  padding-top: 2px;
  padding-bottom: 2px;
  box-shadow: 0 2px 10px var(--ct-color-subtle-copy);
}

.contextMenuOption {
  padding: 5px 15px;
  min-width: 220px;
  font-size: 13px;
  display: flex;
  justify-content: space-between;
  background-color: transparent;
  border: none;
}

.contextMenuOption:hover,
.contextMenuOption:active {
  background-color: var(--ct-color-link);
  color: white;
}
