.package-list-ct {
  margin: 1em 0;
  padding: 0;
  max-width: 110rem;
  text-align: left;
  box-sizing: border-box;
}

.package-list-ct li {
  text-align: left;
  box-sizing: border-box;
  width: 22rem;
  padding: 0 1ex;
  display: inline-block;
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}

.scale-up-ct {
  animation: dialogScaleUpCt 1s;
}

@keyframes dialogScaleUpCt {
  0% {
    opacity: 0;
    max-height: 0;
  }

  25% {
    opacity: 0;
  }

  50% {
    max-height: 100vh;
  }

  100% {
    opacity: 1;
  }
}
