.automation-script-modal pre {
  max-height: 20em;
  margin-bottom: 5px;
}

.automation-script-modal span.fa {
  margin-right: 5px;
}

.automation-script-modal i.fa {
  margin-right: 2px;
  margin-left: 5px;
}

.automation-script-modal textarea {
  min-height: 15rem;
}

.automation-script-modal .ansible-docs-link > svg {
  padding-right: var(--pf-global--spacer--xs);
}

.green-icon {
  color: var(--pf-global--success-color--100);
}

.pf-c-card.modifications-table .pf-c-card__header {
  justify-content: space-between;
}
